@@ -3,22 +3,24 @@ from __future__ import division
 from __future__ import print_function
 
 import itertools
-import json
+import random
 
-from compas.base import Base
+from compas.data import Data
+from compas.datastructures import Mesh
 from compas.files import URDF
-from compas.files import URDFParser
 from compas.files import URDFElement
+from compas.files import URDFParser
 from compas.geometry import Frame
 from compas.geometry import Transformation
+from compas.robots import Configuration
 from compas.robots.model.geometry import Color
 from compas.robots.model.geometry import Geometry
 from compas.robots.model.geometry import Material
 from compas.robots.model.geometry import MeshDescriptor
 from compas.robots.model.geometry import Origin
 from compas.robots.model.geometry import Texture
-from compas.robots.model.geometry import _attr_to_data
 from compas.robots.model.geometry import _attr_from_data
+from compas.robots.model.geometry import _attr_to_data
 from compas.robots.model.joint import Axis
 from compas.robots.model.joint import Joint
 from compas.robots.model.joint import Limit
@@ -28,11 +30,10 @@ from compas.robots.model.link import Visual
 from compas.robots.resources import DefaultMeshLoader
 from compas.topology import shortest_path
 
-
 __all__ = ['RobotModel']
 
 
-class RobotModel(Base):
+class RobotModel(Data):
     """RobotModel is the root element of the model.
 
     Instances of this class represent an entire robot as defined in an URDF
@@ -110,17 +111,6 @@ class RobotModel(Base):
 
         self._rebuild_tree()
 
-    def to_data(self):
-        """Returns the data dictionary that represents the :class:`RobotModel`.
-        To be used in conjunction with :meth:`compas.robot.RobotModel.from_data()`.
-
-        Returns
-        -------
-        dict
-            The RobotModel's data.
-        """
-        return self.data
-
     @classmethod
     def from_data(cls, data):
         """Construct the :class:`compas.robots.RobotModel` from its data representation.
@@ -130,16 +120,6 @@ class RobotModel(Base):
         robot_model.data = data
         return robot_model
 
-    def to_json(self, filepath):
-        with open(filepath, 'w+') as f:
-            json.dump(self.data, f)
-
-    @classmethod
-    def from_json(cls, filepath):
-        with open(filepath, 'r') as fp:
-            data = json.load(fp)
-        return cls.from_data(data)
-
     def _rebuild_tree(self):
         """Store tree structure from link and joint lists."""
         self._adjacency = dict()
@@ -221,6 +201,21 @@ class RobotModel(Base):
         urdf = URDF.from_string(text)
         return urdf.robot
 
+    def to_urdf_string(self, prettify=False):
+        """Construct a URDF string model description from a robot model.
+
+        Parameters
+        ----------
+        prettify:
+            If ``True``, the string will be pretty printed.
+
+        Returns
+        -------
+        :obj:`str`
+        """
+        urdf = URDF.from_robot(self)
+        return urdf.to_string(prettify=prettify)
+
     def find_children_joints(self, link):
         """Returns a list of all children joints of the link.
         """
@@ -472,6 +467,21 @@ class RobotModel(Base):
         joints = self.get_configurable_joints()
         return [joint.type for joint in joints]
 
+    def get_joint_types_by_names(self, names):
+        """Get a list of joint types given a list of joint names.
+
+        Parameters
+        ----------
+        names : :obj:`list` of :obj:`str`
+            The names of the joints.
+
+        Returns
+        -------
+        :obj:`list` of :attr:`compas.robots.Joint.SUPPORTED_TYPES`
+            List of joint types.
+        """
+        return [self.get_joint_by_name(n).type for n in names]
+
     def get_configurable_joint_names(self):
         """Returns the configurable joint names.
 
@@ -537,6 +547,52 @@ class RobotModel(Base):
         joints = self.get_configurable_joints()
         return joints[0].parent.link
 
+    def zero_configuration(self):
+        """Get the zero joint configuration.
+
+        If zero is out of joint limits ``(upper, lower)`` then
+        ``(upper + lower) / 2`` is used as joint value.
+
+        Examples
+        --------
+        >>> robot.zero_configuration()
+        Configuration((0.000, 0.000, 0.000, 0.000, 0.000, 0.000), (0, 0, 0, 0, 0, 0), \
+            ('shoulder_pan_joint', 'shoulder_lift_joint', 'elbow_joint', 'wrist_1_joint', 'wrist_2_joint', 'wrist_3_joint'))
+        """
+        values = []
+        joint_names = []
+        joint_types = []
+        for joint in self.get_configurable_joints():
+            if joint.limit and not (0 <= joint.limit.upper and 0 >= joint.limit.lower):
+                values.append((joint.limit.upper + joint.limit.lower)/2.)
+            else:
+                values.append(0)
+            joint_names.append(joint.name)
+            joint_types.append(joint.type)
+        return Configuration(values, joint_types, joint_names)
+
+    def random_configuration(self):
+        """Get a random configuration.
+
+        Returns
+        -------
+        :class:`compas.robots.Configuration`
+
+        Note
+        ----
+        No collision checking is involved, the configuration may be invalid.
+        """
+        configurable_joints = self.get_configurable_joints()
+        values = []
+        for joint in configurable_joints:
+            if joint.limit:
+                values.append(joint.limit.lower + (joint.limit.upper - joint.limit.lower) * random.random())
+            else:
+                values.append(0)
+        joint_names = self.get_configurable_joint_names()
+        joint_types = self.get_joint_types_by_names(joint_names)
+        return Configuration(values, joint_types, joint_names)
+
     def load_geometry(self, *resource_loaders, **kwargs):
         """Load external geometry resources, such as meshes.
 
@@ -576,6 +632,20 @@ class RobotModel(Base):
                     if not shape.geometry:
                         raise Exception('Unable to load geometry for {}'.format(shape.filename))
 
+    def ensure_geometry(self):
+        """Check if geometry has been loaded.
+        Raises
+        ------
+        :exc:`Exception`
+            If geometry has not been loaded.
+        """
+        for link in self.links:
+            for element in itertools.chain(link.collision, link.visual):
+                shape = element.geometry.shape
+                if not shape.geometry:
+                    raise Exception(
+                        'This method is only callable once the geometry has been loaded.')
+
     @property
     def frames(self):
         """Returns the frames of links that have a visual node.
@@ -588,7 +658,7 @@ class RobotModel(Base):
         frames = []
         for link in self.iter_links():
             if len(link.visual) and link.parent_joint:
-                frames.append(link.parent_joint.origin.copy())
+                frames.append(link.parent_joint.current_origin.copy())
         return frames
 
     @property
@@ -602,8 +672,7 @@ class RobotModel(Base):
         """
         axes = []
         for joint in self.iter_joints():
-            if joint.axis:
-                axes.append(joint.axis.vector)
+            axes.append(joint.current_axis.vector)
         return axes
 
     def __str__(self):
@@ -616,7 +685,7 @@ class RobotModel(Base):
         )
 
     def _create(self, link, parent_transformation):
-        """Private function called during initialisation to transform origins and axes.
+        """Private function called during initialization to transform origins and axes.
 
         Parameters
         ----------
@@ -675,7 +744,7 @@ class RobotModel(Base):
 
         Parameters
         ----------
-        joint_state : dict
+        :class:`compas.robots.Configuration` or joint_state : dict
             A dictionary with the joint names as keys and values in radians and
             meters (depending on the joint type).
         link : :class:`compas.robots.Link`
@@ -690,10 +759,8 @@ class RobotModel(Base):
 
         Examples
         --------
-        >>> names = robot.get_configurable_joint_names()
-        >>> values = [-2.238, -1.153, -2.174, 0.185, 0.667, 0.000]
-        >>> joint_state = dict(zip(names, values))
-        >>> transformations = robot.compute_transformations(joint_state)
+        >>> config = robot.random_configuration()
+        >>> transformations = robot.compute_transformations(config)
         """
         if link is None:
             link = self.root
@@ -723,7 +790,7 @@ class RobotModel(Base):
 
         Parameters
         ----------
-        joint_state : dict
+        :class:`compas.robots.Configuration` or joint_state : dict
             A dictionary with the joint names as keys and values in radians and
             meters (depending on the joint type).
 
@@ -741,14 +808,14 @@ class RobotModel(Base):
         Frame(Point(0.000, 0.000, 0.000), Vector(0.362, 0.932, 0.000), Vector(-0.932, 0.362, 0.000))
         """
         transformations = self.compute_transformations(joint_state)
-        return [j.origin.transformed(transformations[j.name]) for j in self.iter_joints()]
+        return [j.current_origin.transformed(transformations[j.name]) for j in self.iter_joints()]
 
     def transformed_axes(self, joint_state):
         """Returns the transformed axes based on the joint_state.
 
         Parameters
         ----------
-        joint_state : dict
+        :class:`compas.robots.Configuration` or joint_state : dict
             A dictionary with the joint names as keys and values in radians and
             meters (depending on the joint type).
 
@@ -765,14 +832,14 @@ class RobotModel(Base):
         [Vector(0.000, 0.000, 1.000), Vector(0.000, 0.000, 1.000)]
         """
         transformations = self.compute_transformations(joint_state)
-        return [j.axis.transformed(transformations[j.name]) for j in self.iter_joints() if j.axis.vector.length]
+        return [j.current_axis.transformed(transformations[j.name]) for j in self.iter_joints() if j.current_axis.vector.length]
 
     def forward_kinematics(self, joint_state, link_name=None):
         """Calculate the robot's forward kinematic.
 
         Parameters
         ----------
-        joint_state : dict
+        :class:`compas.robots.Configuration` or joint_state : dict
             A dictionary with the joint names as keys and values in radians and
             meters (depending on the joint type).
         link_name : str, optional
@@ -786,10 +853,8 @@ class RobotModel(Base):
 
         Examples
         --------
-        >>> names = robot.get_configurable_joint_names()
-        >>> values = [-2.238, -1.153, -2.174, 0.185, 0.667, 0.000]
-        >>> joint_state = dict(zip(names, values))
-        >>> frame_WCF = robot.forward_kinematics(joint_state)
+        >>> config = robot.random_configuration()
+        >>> frame_WCF = robot.forward_kinematics(config)
         """
         if link_name is None:
             ee_link = self.get_end_effector_link()
@@ -798,7 +863,7 @@ class RobotModel(Base):
         joint = ee_link.parent_joint
         if joint:
             transformations = self.compute_transformations(joint_state)
-            return joint.origin.transformed(transformations[joint.name])
+            return joint.current_origin.transformed(transformations[joint.name])
         else:
             return Frame.worldXY()  # if we ask forward from base link
 
@@ -808,7 +873,13 @@ class RobotModel(Base):
         mesh = kwargs.get(key)
         if mesh:
             meshes.append(mesh)
-        return meshes
+            del kwargs[key]
+        return meshes, kwargs
+
+    def _check_link_name(self, name):
+        all_link_names = [l.name for l in self.links]  # noqa: E741
+        if name in all_link_names:
+            raise ValueError("Link name '%s' already used in chain." % name)
 
     def add_link(self, name, visual_meshes=None, visual_color=None, collision_meshes=None, **kwargs):
         """Adds a link to the robot model.
@@ -819,11 +890,11 @@ class RobotModel(Base):
         ----------
         name : str
             The name of the link
-        visual_meshes : list of :class:`compas.datastructures.Mesh`, optional
+        visual_meshes : list of :class:`compas.datastructures.Mesh` or :class:`compas.geometry.Shape`, optional
             The link's visual mesh.
         visual_color : list of 3 float, optional
             The rgb color of the mesh. Defaults to (0.8, 0.8, 0.8)
-        collision_meshes : list of :class:`compas.datastructures.Mesh`, optional
+        collision_meshes : list of :class:`compas.datastructures.Mesh` or :class:`compas.geometry.Shape`, optional
             The link's collision mesh.
 
         Returns
@@ -843,33 +914,52 @@ class RobotModel(Base):
         >>> robot = RobotModel('robot')
         >>> link = robot.add_link('link0', visual_mesh=mesh)
         """
-        visual_meshes = self._consolidate_meshes(visual_meshes, 'visual_mesh', **kwargs)
-        collision_meshes = self._consolidate_meshes(collision_meshes, 'collision_mesh', **kwargs)
+        self._check_link_name(name)
+        visual_meshes, kwargs = self._consolidate_meshes(visual_meshes, 'visual_mesh', **kwargs)
+        collision_meshes, kwargs = self._consolidate_meshes(collision_meshes, 'collision_mesh', **kwargs)
+        if not visual_color:
+            visual_color = (0.8, 0.8, 0.8)
 
-        all_link_names = [l.name for l in self.links]  # noqa: E741
-        if name in all_link_names:
-            raise ValueError("Link name '%s' already used in chain." % name)
+        visuals = []
+        collisions = []
 
-        visual = []
-        collision = []
-
-        for visual_mesh in visual_meshes:
-            if not visual_color:
-                visual_color = (0.8, 0.8, 0.8)
-            v = Visual(Geometry(MeshDescriptor("")))
+        for visual in visual_meshes:
+            if isinstance(visual, Mesh):
+                v = Visual(Geometry(MeshDescriptor("")))
+                v.geometry.shape.geometry = visual
+            else:
+                v = Visual.from_primitive(visual)
             v.material = Material(color=Color("%f %f %f 1" % visual_color))
-            v.geometry.shape.geometry = visual_mesh
-            visual.append(v)
+            visuals.append(v)
 
-        for collision_mesh in collision_meshes:  # use visual_mesh as collision_mesh if none passed?
-            c = Collision(Geometry(MeshDescriptor("")))
-            c.geometry.shape.geometry = collision_mesh
-            collision.append(c)
+        for collision in collision_meshes:  # use visual_mesh as collision_mesh if none passed?
+            if isinstance(collision, Mesh):
+                c = Collision(Geometry(MeshDescriptor("")))
+                c.geometry.shape.geometry = collision
+            else:
+                c = Collision.from_primitive(collision)
+            collisions.append(c)
 
-        link = Link(name, visual=visual, collision=collision, **kwargs)
+        link = Link(name, visual=visuals, collision=collisions, **kwargs)
         self.links.append(link)
+        # Must build the tree structure, if adding the first link to an empty robot
+        if len(self.links) == 1:
+            self._rebuild_tree()
+            self._create(self.root, Transformation())
         return link
 
+    def remove_link(self, name):
+        """Removes a link to the robot model.
+
+        Provides an easy way to programmatically remove a link from the robot model.
+
+        Parameters
+        ----------
+        name : str
+            The name of the link
+        """
+        self.links = [link for link in self.links if link.name != name]
+
     def add_joint(self, name, type, parent_link, child_link, origin=None, axis=None, limit=None, **kwargs):
         """Adds a joint to the robot model.
 
@@ -945,41 +1035,31 @@ class RobotModel(Base):
         self._joints[joint.name] = joint
         self._adjacency[joint.name] = [child_link.name]
 
-        # Using only part of self._create(link, parent_transformation)
-        parent_transformation = Transformation()
-        for item in itertools.chain(parent_link.visual, parent_link.collision):
-            if not item.init_transformation:
-                item.init_transformation = parent_transformation
-            else:
-                parent_transformation = item.init_transformation
-
-        joint._create(parent_transformation)
-
-        for item in itertools.chain(child_link.visual, child_link.collision):
-            item.init_transformation = joint.current_transformation
+        self._create(self.root, Transformation())
 
         return joint
 
+    def remove_joint(self, name):
+        """Removes a joint to the robot model.
+
+        Provides an easy way to programmatically remove a joint from the robot model.
+
+        Parameters
+        ----------
+        name : str
+            The name of the joint
+        """
+        joint = self.get_joint_by_name(name)
+        self.joints = [j for j in self.joints if j.name != name]
+        parent_link = self.get_link_by_name(joint.parent.link)
+        parent_link.joints = [j for j in parent_link.joints if j.name != name]
+        self._adjacency[parent_link.name] = [j.name for j in parent_link.joints]
+        del self._links[joint.child.link]
+        del self._joints[name]
+        del self._adjacency[name]
+
 
 URDFParser.install_parser(RobotModel, 'robot')
 URDFParser.install_parser(Material, 'robot/material')
 URDFParser.install_parser(Color, 'robot/material/color')
 URDFParser.install_parser(Texture, 'robot/material/texture')
-
-
-if __name__ == '__main__':
-    import os
-    import doctest
-    from compas import HERE
-    from compas.geometry import Sphere  # noqa: F401
-    from compas.robots import GithubPackageMeshLoader  # noqa: F401
-
-    ur5_urdf_file = os.path.join(HERE, '..', '..', 'tests', 'compas', 'robots', 'fixtures', 'ur5.xacro')
-
-    robot = RobotModel("robot", links=[], joints=[])
-    link0 = robot.add_link("world")
-    link1 = robot.add_link("link1")
-    link2 = robot.add_link("link2")
-    robot.add_joint("joint1", Joint.CONTINUOUS, link0, link1, Frame.worldXY(), (0, 0, 1))
-    robot.add_joint("joint2", Joint.CONTINUOUS, link1, link2, Frame.worldXY(), (0, 0, 1))
-    doctest.testmod(globs=globals())
